@@ -7,8 +7,8 @@ import gettext
 import locale
 import tkinter as tk
 from pony.orm import *
-from ..models import funing_m as fm
-from ..setting import base_dir, locale_path, lang_code, setting_yml, setting_path, debug, f_lang_codes
+from fmodel import funing_m as fm
+from setting import base_dir, locale_path, lang_code, setting_yml, setting_path, debug, f_lang_codes
 import yaml
 
 sys_lang_code = locale.getdefaultlocale()[0]\
@@ -21,6 +21,8 @@ if lang_code == 'en-US' and \
     setting_yml['lang_code'] = lang_code
     yaml.dump( setting_yml, open( setting_path, 'w') )
 
+if debug:
+    print(lang_code, locale_path)
 
 lang = gettext.translation(
     'funing',
